// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef GOOGLE_PROTOBUF_ARENA_CONFIG_H__
#define GOOGLE_PROTOBUF_ARENA_CONFIG_H__

#include <atomic>
#include <cstddef>

// Must be included last.
#include "google/protobuf/port_def.inc"

namespace google {
namespace protobuf {
namespace internal {
namespace arena_config_internal {

// We use an atomic here only for correctness so that we can read/write
// concurrently. We don't have memory order requirements so we use relaxed
// memory ordering.
PROTOBUF_EXPORT extern std::atomic<size_t> default_arena_max_block_size;

}  // namespace arena_config_internal

// The default value to use for DefaultArenaMaxBlockSize when
// SetDefaultArenaMaxBlockSize hasn't been called.
PROTOBUF_EXPORT extern const size_t kDefaultDefaultArenaMaxBlockSize;

// The default value to use for arena max block size when no value is provided
// in ArenaOptions.
inline size_t GetDefaultArenaMaxBlockSize() {
  return arena_config_internal::default_arena_max_block_size.load(
      std::memory_order_relaxed);
}
inline void SetDefaultArenaMaxBlockSize(size_t default_arena_max_block_size) {
  return arena_config_internal::default_arena_max_block_size.store(
      default_arena_max_block_size, std::memory_order_relaxed);
}

}  // namespace internal
}  // namespace protobuf
}  // namespace google

#include "google/protobuf/port_undef.inc"

#endif  // GOOGLE_PROTOBUF_ARENA_CONFIG_H__
